class libcore:
    """This is the core library of the package. It contains the main functions to be used by the user."""
    def sum_as_string(self, a: int, b: int) -> int:
        """This function returns the sum of two numbers."""
        ...
